#!/usr/bin/env kross

import re, Kross, KSpread

func = KSpread.function("PYREGEXP")
func.minparam = 3
func.maxparam = 3
func.comment = "The PYREGEXP() function displays the current datetime."
func.syntax = "PYREGEXP(string)"
func.addParameter("String", "The input string.")
func.addParameter("String", "The regular expression.")
func.addParameter("String", "Replace with.")
func.addExample("PYREGEXP(\"Some String\",\"(S|m)\",\"A\")")

def update(args):
    s = args[0]
    regexp = args[1]
    repl = args[2]
    try:
        p = re.compile(regexp)
        func.result = p.sub(repl, s)
    except:
        func.error = "Invalid regexp"

func.connect("called(QVariantList)", update)
func.registerFunction()



#!/usr/bin/env kross

"""
Python script to import content from a Kexi Project stored
within a KexiDB into KSpread.

(C)2007 Sebastian Sauer <mail@dipe.org>
http://kross.dipe.org
http://www.koffice.org/kspread
Dual-licensed under LGPL v2+higher and the BSD license.
"""

#import os, datetime, sys, traceback, csv
#import Kross, KSpread

import sys, os, traceback
import Kross, KSpread

class KexiImport:

    def __init__(self, scriptaction):
        self.scriptaction = scriptaction
        self.currentpath = self.scriptaction.currentPath()
        self.forms = Kross.module("forms")
        self.start()

    def start(self):
        writer = KSpread.writer()

        connection = self.showImportDialog(writer)
        if not connection:
            return

        try:
            print "databaseNames = %s" % connection.databaseNames()
            print "tableNames = %s" % connection.tableNames()
            print "queryNames = %s" % connection.queryNames()

            queryschema = self.showTableDialog(connection)
            if not queryschema:
                return

            print "queryschema.name() = %s" % queryschema.name()
            print "queryschema.caption() = %s" % queryschema.caption()
            print "queryschema.description() = %s" % queryschema.description()

            cursor = connection.executeQuerySchema(queryschema)
            if not cursor:
                raise "Failed to create cursor."
            if not cursor.moveFirst():
                raise "The cursor has no records to read from."

            while not cursor.eof():
                record = []
                for i in range( cursor.fieldCount() ):
                    record.append( cursor.value(i) )
                if writer.setValues(record):
                    writer.next()
                else:
                    print "Failed to set all of '%s' to cell '%s'" % (record,writer.cell())
                cursor.moveNext()
        finally:
            connection.disconnect()

    def createConnection(self, projectfile):
        kexidb = Kross.module("kexidb")
        if not kexidb:
            raise "Failed to load the KexiDB Scripting module. This script needs Kexi to run."
        connectiondata = kexidb.createConnectionDataByFile(projectfile)
        if not connectiondata:
            raise "Invalid Kexi Project File: %s" % projectfile
        connectiondata.setFileName(projectfile)
        connectiondata.setDatabaseName(projectfile)
        driver = kexidb.driver(connectiondata.driverName())
        if not driver:
            raise "No KexiDB driver for: %s" % connectiondata.driverName()
        connection = driver.createConnection(connectiondata)
        if not connection.connect():
            raise "Failed to connect: %s" % connection.lastError()
        if not connection.useDatabase(connectiondata.databaseName()):
            raise "Failed to open database: %s" % connection.lastError()
        return connection

    def showImportDialog(self, writer):
        dialog = self.forms.createDialog("Kexi Import")
        dialog.setButtons("Ok|Cancel")
        dialog.setFaceType("List") #Auto Plain List Tree Tabbed

        openpage = dialog.addPage("Open","Import from Kexi Project File","document-open")
        openwidget = self.forms.createFileWidget(openpage, "kfiledialog:///kspreadkexidbimportopen")
        openwidget.setMode("Opening")
        openwidget.setFilter("*.kexi *.kexis *kexic|Kexi Project Files\n*|All Files")

        datapage = dialog.addPage("Import","Import to sheet beginning at cell","document-import")
        sheetslistview = KSpread.createSheetsListView(datapage)
        sheetslistview.setEditorType("Cell")

        if dialog.exec_loop():
            projectfile = openwidget.selectedFile()
            if not os.path.isfile(projectfile):
                raise "File '%s' not found." % projectfile

            currentSheet = sheetslistview.sheet()
            if not currentSheet:
                raise "No current sheet."
            if not writer.setSheet(currentSheet):
                raise "Invalid sheet \"%s\" defined." % currentSheet

            cell = sheetslistview.editor()
            if not writer.setCell(cell):
                raise "Invalid cell \"%s\" defined." % cell

            connection = self.createConnection(projectfile)
            return connection
        return None

    def showTableDialog(self, connection):
        tabledialog = self.forms.createDialog("Table or Query")
        tabledialog.setButtons("Ok|Cancel")
        tabledialog.setFaceType("List") #Auto Plain List Tree Tabbed
        sourcepage = tabledialog.addPage("Data","Datasource","table")
        items = []
        for s in connection.tableNames(): items.append("table/%s" % s)
        for s in connection.queryNames(): items.append("query/%s" % s)
        sourcewidget = self.forms.createWidgetFromUI(sourcepage,
            '<ui version="4.0" >'
            ' <class>Form</class>'
            ' <widget class="QWidget" name="Form" >'
            '  <layout class="QHBoxLayout" >'
            '   <item>'
            '    <widget class="QListWidget" name="List">'
            '     <property name="currentRow"><number>%i</number></property>'
            '     %s'
            '    </widget>'
            '   </item>'
            '  </layout>'
            ' </widget>'
            '</ui>'
            % ( 0 , ''.join([ '<item><property name="text" ><string>%s</string></property></item>' % s for s in items ]) )
        )
        #print dir(widget)
        self.sourcelist = sourcewidget["List"]
        if tabledialog.exec_loop():
            current = items[ self.sourcelist.currentRow ]
            if current.startswith("table/"):
                schema = connection.tableSchema(current[6:])
                if not schema:
                    raise "No such tableschema \"%s\"" % current[6:]
                return schema.query()
            elif current.startswith("query/"):
                print "QUERY ==============> %s" % current[6:]
                schema = connection.querySchema(current[6:])
                if not schema:
                    raise "No such queryschema \"%s\"" % current[6:]
                return schema
            else:
                raise "Unknown item %s" % current
        return None

        #csvfilename = self.openwidget.selectedFile()
        #if not os.path.isfile(csvfilename):
            #raise "File '%s' not found." % csvfilename
        ##writer.connect("valueChanged()",writer.next)
        #csv.register_dialect("custom", self.getCustomDialect())
        #csvfile = open(csvfilename,'r')
        #try:
            #csvreader = csv.reader(csvfile, dialect="custom")
            #try:
                #while True:
                    #record = csvreader.next()
                    #if not writer.setValues(record):
                        #print "Failed to set all of '%s' to cell '%s'" % (record,writer.cell())
                    ##writer.insertValues(record)
                    #writer.next()
            #except StopIteration:
                #pass
        #finally:
            #csvfile.close()

KexiImport( self )



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

"""
This python script connects KWord and OpenOffice.org together.

(C)2006 Sebastian Sauer <mail@dipe.org>

http://kross.dipe.org
http://www.koffice.org/kword
http://udk.openoffice.org/python/python-bridge.html

Dual-licensed under LGPL v2+higher and the BSD license.
"""

import sys, os, popen2, time, traceback, tempfile
import Kross, KWord

class Importer:
    def __init__(self, action):
        self.action = action
        self.forms = Kross.module("forms")
        self.pyfile = os.path.join(action.currentPath(), 'oouno.py')
        if not os.path.isfile(self.pyfile):
            raise "Failed to find the required oouno.py file."

    def showDialog(self):
        dialog = self.forms.createDialog("Import file with OpenOffice.org")
        dialog.setButtons("Ok|Cancel")
        dialog.setFaceType("List") #Auto Plain List Tree Tabbed

        openpage = dialog.addPage("Open","Import File","document-open")
        startDirOrVariable = "kfiledialog:///kwordooimport"
        openwidget = self.forms.createFileWidget(openpage, startDirOrVariable)
        openwidget.setMode("Opening")
        filters = [
            "*.odt|OpenDocument Text (*.odt)",
            "*.ott|OpenDocument Text Template (*.ott)",
            "*.sxw|OpenOffice.org 1.0 Text Document (*.sxw)",
            "*.stw|OpenOffice.org 1.0 Text Document Template (*.stw)",
            "*.doc|Microsoft Word 95/97/2000/XP (*.doc)",
            "*.dot|Microsoft Word 95/97/2000/XP Template (*.dot)",
            "*.xml|Microsoft Word 2003 XML (*.xml)",
            "*.rtf|Rich Text Format (*.rtf)",
            "*.txt|Text (*.txt)",
            "*.html *.htm|HTML Document (*.html *.htm)",
            "*.xml|DocBook (*.xml)",
            "*.sdw|StarWriter 1.0 - 5.0 (*.sdw)",
            "*.vor|StarWriter 3.0 - 5.0 Templates (*.vor)",
            "*.wpd|WordPerfect Document (*.wpd)",
            "*.lwp|Lotus WordPro Document (*.lwp)",
            "*.jtd|Ichitaro 8/9/10/11 (*.jtd)",
            "*.jtt|Ichitaro 8/9/10/11 Template (*.jtt)",
            "*.hwp|Hangul WP 97 (*.hwp)",
            "*.wps|WPS 2000/Office 1.0 (*.wps)",
        ]
        filters.insert(0, "%s|All Supported Files" % " ".join( [f.split("|")[0] for f in filters] ))
        filters.append("*|All Files")
        openwidget.setFilter("\n".join(filters))

        configpage = dialog.addPage("Connect","OpenOffice.org UNO Connection","network-server")
        configwidget = self.forms.createWidgetFromUIFile(configpage, os.path.join(self.action.currentPath(),"ooimportconfig.ui"))

        importpage = dialog.addPage("Options","Import Options","preferences-other")
        importwidget = self.forms.createWidgetFromUIFile(importpage, os.path.join(self.action.currentPath(),"ooimport.ui"))

        while True:
            if not dialog.exec_loop():
                #dialog.delayedDestruct()
                return None
            loadfile = openwidget.selectedFile()
            if loadfile and os.path.isfile(loadfile):
                break
            self.forms.showMessageBox("Error","Error","No such file \"%s\"" % loadfile)

        options = {}
        options["loadfile"] = loadfile
        options["host"] = configwidget["hostEdit"].text
        options["port"] = configwidget["portEdit"].value
        if configwidget["timeoutCheckBox"].checked:
            options["timeout"] = configwidget["timeoutEdit"].value
        else:
            options["timeout"] = -1
        options["startupServer"] = configwidget["startupCheckBox"].checked
        options["hideClient"] = configwidget["hideClientCheckBox"].checked
        options["autoCloseDocument"] = configwidget["autoCloseCheckBox"].checked

        if importwidget["odtRadioButton"].checked:
            options["_savefileext"] = '.odt'
        elif importwidget["textRadioButton"].checked:
            options["_savefileext"] = '.txt'
        elif importwidget["htmlRadioButton"].checked:
            options["_savefileext"] = '.html'
        else:
            raise "Invalid option for the intermedia format"

        return options

    def doImport(self, options):
        savefile = tempfile.mktemp()
        savefile += options["_savefileext"]
        options["savefile"] = savefile

        progress = self.forms.showProgressDialog("Import...", options["loadfile"])
        progress.setRange(0,150)
        progress.setValue(0)
        canceled = False
        step = 0
        try:
            args = ["python", self.pyfile]
            args += [ "--%s=%s" % (n,options[n]) for n in options.keys() if not n.startswith('_') ]
            print "Execute: ", args, "\n"
            proc = popen2.Popen3( args, capturestderr = 1 )
            errmsg = proc.childerr.read()
            if errmsg:
                progress.addText("<b>%s</b>" % errmsg)
            fromchild = proc.fromchild.readline()
            while fromchild:
                step += 1
                progress.setValue(step)
                if fromchild and fromchild.strip() != "":
                    progress.addText(fromchild)
                sys.stdout.write( fromchild )
                sys.stdout.flush()

                errmsg = proc.childerr.read()
                if errmsg:
                    progress.addText("<b>%s</b>" % errmsg)
                canceled = progress.isCanceled()
                if canceled:
                    break
                #time.sleep(1)
                fromchild = proc.fromchild.readline()

            if not canceled:
                if not os.path.isfile(savefile):
                    if not proc.childerr:
                        progress.addText("<b>Error: Seems OpenOffice.org failed to write the intermedia file \"%s\"</b>" % savefile)
                    canceled = True
                else:
                    progress.addText("KWord imports the intermedia file \"%s\" now" % savefile)
                    self.importFile(savefile, options)
        finally:
            try:
                if os.path.isfile(savefile):
                    os.remove(savefile)
            except:
                pass
            if canceled:
                progress.addText("<br><b>Import aborted.</b>")
            else:
                progress.addText("<br><b>Import successful finished.</b>")
            progress.exec_loop()

        #if self.dialog.exec_loop():
            #try:
                #file = openwidget.selectedFile()
                #if not file or not os.path.isfile(file):
                    #raise "No such file: %s" % file
                #progressThread = ProgressThread(self.forms, file)
                #progressThread.start()
                #try:
                    #controller = UnoController()
                    #progressThread.value = 5
                    #controller.connect()
                    #try:
                        #progressThread.value = 30
                        #controller.loadDocument( "file://%s" % file )
                        #progressThread.value = 70
                        #outputstream = KWordOutputStream(controller.unoConfig)
                        #controller.writeDocument(outputstream)
                        #progressThread.value = 90
                        #outputstream.flush()
                    #finally:
                        #controller.disconnect()
                #finally:
                    #progressThread.finish()
                    #progressThread.join() # wait till the thread finished
            #except:
                #tb = "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
                #self.forms.showMessageBox("Error","Error","%s" % tb)

    def importFile(self, fromFileName, options):
        fs = KWord.mainFrameSet()
        if not fs:
            raise "No main frameset to import the content too."
        doc = fs.document()
        if not fs:
            raise "No document to import the content too."

        fileExt = os.path.splitext(fromFileName)[1].lower()
        if fileExt == '.txt' or fileExt == '.html':
            f = open(fromFileName, "r")
            if fileExt == '.txt':
                doc.setText( ''.join(f.readlines()) )
            else:
                doc.setHtml( ' '.join(f.readlines()) )
            f.close()
        else: #odt

            #TODO this crashes horrible :-(
            KWord.document().openUrl(fromFileName)

def start(action):
    importer = Importer(action)
    options = importer.showDialog()
    if options != None:
        importer.doImport(options)

start(self)

##print "CONNECT ..."
##controller.connect()
##print "LOADING DOC ..."
##controller.loadDocument("file:///home/kde4/bla2.odt")
###controller.loadDocument("file:///home/kde4/bla.odt")
##print "DUMP DOC ..."
##controller.writeDocument()
##print "DISCONNECT ..."
##controller.disconnect()
##print "DONE"





#!/usr/bin/env kross

"""
Python script to export content from KSpread to a
Kexi Project stored within a KexiDB.

(C)2007 Sebastian Sauer <mail@dipe.org>
http://kross.dipe.org
http://www.koffice.org/kspread
Dual-licensed under LGPL v2+higher and the BSD license.
"""

#import os, datetime, sys, traceback, csv
#import Kross, KSpread

import sys, os, traceback
import Kross, KSpread

class KexiExport:

    def __init__(self, scriptaction):
        self.scriptaction = scriptaction
        self.currentpath = self.scriptaction.currentPath()
        self.forms = Kross.module("forms")
        self.start()

    def start(self):
        reader = KSpread.reader()

        connection = self.showExportDialog(reader)
        if not connection:
            return

        try:
            print "databaseNames = %s" % connection.databaseNames()
            print "tableNames = %s" % connection.tableNames()

            tableschema = self.showTableDialog(connection)
            if not tableschema:
                return

            fieldlist = tableschema.fieldlist()

            print "tableschema.name() = %s" % tableschema.name()
            print "tableschema.caption() = %s" % tableschema.caption()
            print "tableschema.description() = %s" % tableschema.description()
            print "fieldlist.names() = %s" % fieldlist.names()

            def changedSheet(sheetname):
                print "changedSheet sheetname=%s" % sheetname

            def changedRow(row):
                values = reader.currentValues()
                #values = [ "%s" % v for v in reader.currentValues() ]

                print "changedRow row=%i values=%s" % (row,values)
                if connection.insertRecord(fieldlist, values):
                    pass
                else:
                    print "=> insert failed: %s" % connection.lastError()

            reader.connect("changedSheet(QString)",changedSheet)
            reader.connect("changedRow(int)",changedRow)
            reader.start()

        finally:
            connection.disconnect()

    def createConnection(self, projectfile):
        kexidb = Kross.module("kexidb")
        if not kexidb:
            raise "Failed to load the KexiDB Scripting module. This script needs Kexi to run."
        connectiondata = kexidb.createConnectionDataByFile(projectfile)
        if not connectiondata:
            raise "Invalid Kexi Project File: %s" % projectfile
        connectiondata.setFileName(projectfile)
        connectiondata.setDatabaseName(projectfile)
        driver = kexidb.driver(connectiondata.driverName())
        if not driver:
            raise "No KexiDB driver for: %s" % connectiondata.driverName()
        connection = driver.createConnection(connectiondata)
        if not connection.connect():
            raise "Failed to connect: %s" % connection.lastError()
        if not connection.useDatabase(connectiondata.databaseName()):
            raise "Failed to open database: %s" % connection.lastError()
        return connection

    def showExportDialog(self, reader):
        dialog = self.forms.createDialog("Kexi Export")
        dialog.setButtons("Ok|Cancel")
        dialog.setFaceType("List") #Auto Plain List Tree Tabbed

        savepage = dialog.addPage("Save","Export to Kexi Project File","document-save")
        savewidget = self.forms.createFileWidget(savepage, "kfiledialog:///kspreadkexidbexport")
        savewidget.setMode("Saving")
        savewidget.setFilter("*.kexi *.kexis *kexic|Kexi Project Files\n*|All Files")

        datapage = dialog.addPage("Export","Export sheets and ranges","document-export")
        sheetslistview = KSpread.createSheetsListView(datapage)
        sheetslistview.setSelectionType("MultiSelect")
        sheetslistview.setEditorType("Range")

        if dialog.exec_loop():
            projectfile = savewidget.selectedFile()
            if not os.path.isfile(projectfile):
                raise "File '%s' not found." % projectfile
            reader.setSheets( sheetslistview.sheets() )
            connection = self.createConnection(projectfile)
            return connection
        return None

    def showTableDialog(self, connection):
        tabledialog = self.forms.createDialog("Table or Query")
        tabledialog.setButtons("Ok|Cancel")
        tabledialog.setFaceType("List") #Auto Plain List Tree Tabbed
        sourcepage = tabledialog.addPage("Data","Datasource","table")
        items = connection.tableNames()
        sourcewidget = self.forms.createWidgetFromUI(sourcepage,
            '<ui version="4.0" >'
            ' <class>Form</class>'
            ' <widget class="QWidget" name="Form" >'
            '  <layout class="QHBoxLayout" >'
            '   <item>'
            '    <widget class="QListWidget" name="List">'
            '     <property name="currentRow"><number>%i</number></property>'
            '     %s'
            '    </widget>'
            '   </item>'
            '  </layout>'
            ' </widget>'
            '</ui>'
            % ( 0 , ''.join([ '<item><property name="text" ><string>%s</string></property></item>' % s for s in items ]) )
        )
        self.sourcelist = sourcewidget["List"]
        if tabledialog.exec_loop():
            current = items[ self.sourcelist.currentRow ]
            return connection.tableSchema(current)
        return None

KexiExport( self )



#!/usr/bin/env kross

import time, Kross, KSpread

func = KSpread.function("PYTIME")
func.minparam = 0
func.maxparam = 1
func.comment = "The PYTIME() function displays the current datetime."
func.syntax = "PYTIME(string)"
func.addParameter("String", "The datetime format string.")
func.addExample("PYTIME()")
func.addExample("PYTIME(\"%H:%M.%S\")")
func.addExample("PYTIME(\"%Y-%M-%d\")")

def update(args):
    try:
        func.result = time.strftime(args[0] or "%H:%M.%S")
    except:
        func.error = "Invalid format"

func.connect("called(QVariantList)", update)
func.registerFunction()



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import KWord, time

#KWord.insertPage( KWord.pageCount() )
#if KWord.pageCount() < 1: KWord.insertPage(0)
doc = KWord.mainFrameSet().document()

cursor = doc.rootFrame().lastCursorPosition()
#cursor = doc.lastCursor()

variables = doc.variableNames()

cursor.insertDefaultBlock()
cursor.insertHtml("<b>Variables with Cursor</b><br>")
for n in variables:
    cursor.insertHtml("%s: " % n)
    ok = doc.addVariable(cursor,n)
    cursor.insertDefaultBlock()
    print "VARIABLE ADDED =====> variablename=%s ok=%s" % (n,ok)

for n in variables:
    doc.setVariableValue(n, "%s" % time.strftime('%H:%M.%S'))

cursor.insertDefaultBlock()
cursor.insertHtml("<b>Variables with HTML</b><br>")
cursor.insertHtml("<p>%s</p>" % "".join([ "%s: %s<br>" % (v,doc.variableValue(v)) for v in variables ]))



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import KWord

idx = 0
shapes = KWord.shapeKeys()
for shapeId in shapes:
    frame = KWord.addFrame("myshape", shapeId)
    if frame != None:
        #frame.setTextRunAround(tableframe.RunThrough)
        frame.setPosition(idx % 6 * 80 + 20, 100 * idx + 20)
        #frame.resize(160, 160)
        idx += 1
